local addonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensGlobalTables.GetTable("BittensSpellFlashLibrary")

local CheckInteractDistance = CheckInteractDistance
local GetTime = GetTime
local IsItemInRange = IsItemInRange
local math = math
local pairs = pairs

------------------------------------------------------------------------ Common
c.AddOptionalSpell("Power Word: Fortitude", nil, {
	NoRangeCheck = true,
	CheckFirst = function()
		return c.RaidBuffNeeded(c.STAMINA_BUFFS)
	end
})

c.AddOptionalSpell("Inner Fire", nil, {
	Buff = "Inner Fire",
	BuffUnit = "player",
})

c.AddOptionalSpell("Power Infusion", nil, {
	NoGCD = true,
	NoRangeCheck = true,
})

c.AddOptionalSpell("Flash Heal", "under Surge of Light", {
	NoRangeCheck = true,
	NoStopChannel = true,
	CheckFirst = function()
		return c.HasBuff("Surge of Light")
	end
})

c.AddOptionalSpell("Shadowfiend", "for Mana", {
	NoStopChannel = true,
    CheckFirst = function()
        return s.PowerPercent("player") < 76
    end
})

c.AddOptionalSpell("Mindbender", "for Mana", {
	NoStopChannel = true,
	CheckFirst = function()
		return s.PowerPercent("player") < 85
	end
})

c.AddOptionalSpell("Inner Fire", "or Will", {
	FlashID = { "Inner Fire", "Inner Will" },
	CheckFirst = function()
		return not c.HasBuff("Inner Fire") and not c.HasBuff("Inner Will")
	end
})

c.AddOptionalSpell("Desperate Prayer", nil, {
	Override = function()
		return s.HealthPercent("player") < 70
			and s.HasSpell(c.GetID("Desperate Prayer"))
			and c.GetCooldown("Desperate Prayer") == 0
	end
})

c.AddOptionalSpell("Prayer of Mending", nil, {
	NoRangeCheck = true,
	NoStopChannel = true,
	CheckFirst = function()
		return not s.MyBuff(
			c.GetID("Prayer of Mending"), 
			a.PrayerOfMendingTarget,
			c.GetBusyTime())
	end
})

c.AddOptionalSpell("Binding Heal", nil, {
	NoRangeCheck = true,
	NoStopChannel = true,
	CheckFirst = function()
		return s.HealthPercent("player") < 85
	end
})

-------------------------------------------------------------------- Discipline
c.RegisterForFullChannels("Penance", 2)

c.AddOptionalSpell("Inner Focus", nil, {
	NoStopChannel = true,
})

c.AddOptionalSpell("Penance", "under Borrowed Time", {
	NoRangeCheck = true,
	NoStopChannel = true,
	CheckFirst = function()
		return c.HasBuff("Borrowed Time")
	end
})

c.AddOptionalSpell("Power Word: Shield", "under Divine Insight", {
	NoRangeCheck = true,
	NoStopChannel = true,
	CheckFirst = function()
		return c.HasBuff("Divine Insight")
	end
})

c.AddOptionalSpell("Archangel", nil, {
	NoStopChannel = true,
	CheckFirst = function(z)
		local dur = c.GetBuffDuration("Evangelism")
		local stack = c.GetBuffStack("Evangelism")
		if c.IsCasting("Penance", "Smite", "Holy Fire") then
			if dur > 0 then
				stack = math.min(5, stack + 1)
			else
				stack = 1
			end
			dur = 20
		end
		
		if dur == 0 or dur > 5 then
			return false
		end
		
		if stack < 5 then
			z.FlashSize = s.FlashSizePercent() / 2
		else
			z.FlashSize = nil
		end
		if dur < 1 then
			z.FlashColor = "red"
		else
			z.FlashColor = "yellow"
		end
		return true
	end
})

------------------------------------------------------------------------ Shadow
c.AddOptionalSpell("Shadowform", nil, {
	Type = "form",
})

c.AddOptionalSpell("Shadowfiend", nil, {
	Override = function()
		return c.GetCooldown("Shadowfiend") == 0
	end
})

c.AddOptionalSpell("Mindbender", nil, {
	Override = function()
		return c.GetCooldown("Mindbender") == 0
	end
})

c.AddOptionalSpell("Vampiric Embrace", nil, {
	CheckFirst = function()
		return c.IsSolo() and c.GetHealthPercent("player") < 50
	end
})

c.AddInterrupt("Silence", nil, {
	NoGCD = true,
})

c.AddSpell("Mind Blast", nil, {
	GetDelay = function()
		return a.Orbs < 3 and c.GetCooldown("Mind Blast", false, 8)
	end
})

c.AddSpell("Mind Blast", "Delay", {
	IsMinDelayDefinition = true,
	GetDelay = function()
		return a.Orbs < 3 and c.GetCooldown("Mind Blast", false, 8), .5
	end,
})

c.AddSpell("Shadow Word: Death", nil, {
	GetDelay = function()
		if not a.InExecute then
			return false
		end
		
		if a.SWDinARow == 1 then
			return 0
		end
		
		return c.GetCooldown("Shadow Word: Death", false, 8)
	end,
})

c.AddSpell("Shadow Word: Death", "for Orb", {
	GetDelay = function()
		if not a.InExecute or a.Orbs == 3 then
			return false
		end
		
		if a.SWDinARow == 0 then
			return c.GetCooldown("Shadow Word: Death", false, 8)
		elseif a.SWDinARow == 1 then
			return 6 - a.SinceSWD
		end
	end,
})

c.AddSpell("Shadow Word: Death", "without Orb", {
	GetDelay = function()
		if a.InExecute and a.SWDinARow == 1 then
			return 0, 2.5 - a.SinceSWD
		end
	end,
})

c.MakePredictor(c.AddSpell("Shadow Word: Death", "Delay", {
	GetDelay = function()
		if not a.InExecute then
			return false
		end
		
		if a.SWDInARow == 0 then
			return c.GetCooldown("Shadow Word: Death", false, 8) - .5
		elseif a.SWDInARow == 1 then
			return 5.5 - a.SinceSWD
		end
	end,
}))

c.AddSpell("Mind Flay", nil, {
	CanCastWhileMoving = false,
	GetDelay = function(z)
		local delay = c.GetMyDebuffDuration("Mind Flay (Insanity)")
		if delay > 0 then
			z.WhiteFlashOffset = 0
			return delay
		end
		
		z.WhiteFlashOffset = -a.FlayTick
		return s.GetChanneling(z.ID, "player") - c.GetBusyTime()
	end
})

c.AddSpell("Mind Flay", "(Insanity)", {
	CanCastWhileMoving = false,
	GetDelay = function(z)
		if not c.HasTalent("Solace and Insanity") then
			return false
		end
		
		local plague = c.GetMyDebuffDuration(
			"Devouring Plague", false, false, true)
		if plague == 0 then
			return false
		end
		
		if a.Insanity == 0 then
			z.WhiteFlashOffset = 0
			return 0
		end
		
		if a.Insanity - a.FlayTick > plague - .5 then
			z.WhiteFlashOffset = -2 * a.FlayTick
		else
			z.WhiteFlashOffset = -a.FlayTick
		end
		return a.Insanity
	end,
})
c.RegisterInitiatingSpell("Mind Flay", "Mind Flay (Insanity)")

c.AddSpell("Mind Flay", "(Insanity) Delay", {
	IsMinDelayDefinition = true,
	GetDelay = function()
		return a.Insanity
	end,
})

c.AddSpell("Shadow Word: Pain", nil, {
	MyDebuff = "Shadow Word: Pain",
})
c.ManageDotRefresh("Shadow Word: Pain", 3)

c.AddSpell("Shadow Word: Pain", "Application", {
	MyDebuff = "Shadow Word: Pain",
})

c.AddSpell("Shadow Word: Pain", "Early", {
	MyDebuff = "Shadow Word: Pain",
	CheckFirst = function()
		return a.InExecute and c.HasTalent("Solace and Insanity")
	end
})
c.ManageDotRefresh("Shadow Word: Pain Early", 6, "Shadow Word: Pain")

c.AddSpell("Shadow Word: Pain", "Moving", {
	MyDebuff = "Shadow Word: Pain",
	EarlyRefresh = 15,
})

c.AddSpell("Vampiric Touch", nil, {
	MyDebuff = "Vampiric Touch",
})
c.ManageDotRefresh("Vampiric Touch", 3)

c.AddSpell("Vampiric Touch", "Application", {
	MyDebuff = "Vampiric Touch",
})

c.AddSpell("Vampiric Touch", "Early", {
	MyDebuff = "Vampiric Touch",
	CheckFirst = function()
		return a.InExecute and c.HasTalent("Solace and Insanity")
	end
})
c.ManageDotRefresh("Vampiric Touch Early", 6, "Vampiric Touch")

c.AddSpell("Mind Spike", "under Surge of Darkness", {
	CheckFirst = function()
		return a.Surges > 0
	end
})

c.AddSpell("Mind Spike", "under Surge of Darkness Cap", {
	CheckFirst = function()
		return a.Surges == 2
	end
})

c.AddSpell("Devouring Plague", nil, {
	MyDebuff = "Devouring Plague",
	CheckFirst = function()
		return a.Orbs == 3
	end,
})
c.ManageDotRefresh("Devouring Plague", 1)

c.AddSpell("Cascade", nil, {
	GetDelay = function(z)
		local dist = c.DistanceAtTheLeast()
		if dist >= 40 then
			return false
		elseif dist >= 30 then
			z.FlashColor = "green"
		elseif dist >= 20 then
			z.FlashColor = "yellow"
		else
			z.FlashColor = "red"
		end
		return c.GetCooldown("Cascade", false, 25)
	end
})

c.AddOptionalSpell("Divine Star", nil, {
	GetDelay = function(z)
		local dist = c.DistanceAtTheMost()
		if dist > 24 then
			z.FlashColor = "red"
		else
			z.FlashColor = "yellow"
		end
		return c.GetCooldown("Divine Star", false, 15)
	end
})

c.AddOptionalSpell("Halo", nil, {
	GetDelay = function(z)
		z.FlashSize = nil
		local dist = c.DistanceAtTheLeast()
		if dist >= 25 then
			z.FlashColor = "red"
		elseif dist >= 20 then
			if not CheckInteractDistance(s.UnitSelection(), 1) then
				z.FlashSize = s.FlashSizePercent() / 2
			end
			z.FlashColor = "green"
		elseif dist >= 15 then
			z.FlashColor = "yellow"
		else
			z.FlashColor = "red"
		end
		return c.GetCooldown("Halo", false, 40)
	end
})

c.AddOptionalSpell("Dispersion", nil, {
	CheckFirst = function()
		return s.PowerPercent("player") < 64
	end
})
